from discord.ext import commands

from .errors import MustBeSameChannel, NotConnectedToVoice, PlayerNotConnected


def voice_connected():
    def predicate(ctx: commands.Context):
        if not ctx.author.voice:
            raise NotConnectedToVoice("You are not connected to any voice channel.")

        return True

    return commands.check(predicate)


def voice_channel_player():
    def predicate(ctx: commands.Context):
        if not ctx.author.voice:
            raise NotConnectedToVoice("You are not connected to any voice channel.")

        if not ctx.voice_client:
            raise PlayerNotConnected("Player is not connected to any voice channel.")

        if ctx.voice_client.channel.id != ctx.author.voice.channel.id:
            raise MustBeSameChannel(
                "You must be in the same voice channel as the player."
            )

        return True

    return commands.check(predicate)



import asyncio
import math

from discord import Color, Embed, Forbidden, InvalidArgument, NotFound, HTTPException

from ._emojis import Emojis


class Paginator:
    def __init__(self, ctx, player) -> None:
        self.ctx = ctx
        self.player = player

    @staticmethod
    def get_length(queue):
        length = sum([track.length for track in queue._queue])
        if length > 3600:
            length = f"{int(length // 3600)}h {int(length % 3600 // 60)}m {int(length % 60)}s"
        elif length > 60:
            length = f"{int(length // 60)}m {int(length % 60)}s"
        else:
            length = f"{int(length)}s"

        return length

    def create_embed(self, tracks, current_page, total_pages):
        embed = Embed(color=Color(0x2F3136))
        embed.set_author(
            name="Queue",
            icon_url="https://cdn.discordapp.com/attachments/776345413132877854/940247400046542948/list.png",
        )

        if self.player.loop == "CURRENT":
            next_song = (
                f"Next > [{self.player.source.title}]({self.player.source.uri}) \n\n"
            )
        else:
            next_song = ""

        description = next_song
        queue_length = self.get_length(self.player.queue)

        for index, track in enumerate(tracks):
            description += (
                f"{current_page * 10 + index + 1}. [{track.title}]({track.uri}) \n"
            )

        embed.description = description

        if total_pages == 1:
            embed.set_footer(
                text=f"{len(self.player.queue._queue)} tracks, {queue_length}"
            )
        else:
            embed.set_footer(
                text=f"Page {current_page + 1}/{total_pages}, {len(self.player.queue._queue)} tracks, {queue_length}"
            )

        return embed

    async def start(self):
        per_page = 10
        current_page = 0
        track_list = list(self.player.queue._queue)

        total_pages = math.ceil(len(track_list) / per_page)

        msg = None

        while True:
            tracks = track_list[current_page * per_page : (current_page + 1) * per_page]
            embed = self.create_embed(tracks, current_page, total_pages)

            if not msg:
                msg = await self.ctx.send(embed=embed)
            else:
                await msg.edit(embed=embed)

            if total_pages > 1:
                try:
                    await msg.add_reaction(Emojis.FIRST)
                    await msg.add_reaction(Emojis.PREV)
                    await msg.add_reaction(Emojis.NEXT)
                    await msg.add_reaction(Emojis.LAST)
                except (HTTPException, Forbidden, NotFound, InvalidArgument) as e:
                    print(e)
                    pass
            else:
                break

            def check(reaction, user):
                valid_reactions = [Emojis.FIRST, Emojis.PREV, Emojis.NEXT, Emojis.LAST]
                return (
                    user == self.ctx.author
                    and str(reaction.emoji) in valid_reactions
                    and reaction.message.id == msg.id
                )

            try:
                reaction, user = await self.ctx.bot.wait_for(
                    "reaction_add", timeout=60.0, check=check
                )
            except asyncio.TimeoutError:
                break

            if str(reaction.emoji) == Emojis.PREV:
                current_page = max(0, current_page - 1)
            elif str(reaction.emoji) == Emojis.NEXT:
                current_page = min(total_pages - 1, current_page + 1)
            elif str(reaction.emoji) == Emojis.FIRST:
                current_page = 0
            elif str(reaction.emoji) == Emojis.LAST:
                current_page = total_pages - 1

            await msg.remove_reaction(reaction.emoji, user)



from typing import Union

from wavelink import SoundCloudTrack, YouTubeMusicTrack, YouTubeTrack, YouTubePlaylist
from wavelink.ext.spotify import SpotifyTrack

Provider = Union[
    YouTubeTrack, YouTubePlaylist, YouTubeMusicTrack, SoundCloudTrack, SpotifyTrack
]



from discord.ext.commands.errors import CheckFailure


class NotConnectedToVoice(CheckFailure):
    """User not connected to any voice channel"""

    pass


class PlayerNotConnected(CheckFailure):
    """Player not connected"""

    pass


class MustBeSameChannel(CheckFailure):
    """Player and user not in same channel"""

    pass


class NothingIsPlaying(CheckFailure):
    """Nothing is playing"""

    pass


class NotEnoughSong(CheckFailure):
    """Not enough songs in queue"""

    pass


class InvalidLoopMode(CheckFailure):
    """Invalid loop mode"""

    pass



import wavelink
from discord.ext import commands

from .errors import (
    InvalidLoopMode,
    MustBeSameChannel,
    NotConnectedToVoice,
    NotEnoughSong,
    NothingIsPlaying,
    PlayerNotConnected,
)
from .player import DisPlayer


class MusicEvents(commands.Cog):
    def __init__(self, bot) -> None:
        self.bot = bot

    async def handle_end_stuck_exception(
        self, player: DisPlayer, track: wavelink.abc.Playable
    ):
        if player.loop == "CURRENT":
            return await player.play(track)

        if player.loop == "PLAYLIST":
            await player.queue.put(track)

        player._source = None
        await player.do_next()

    @commands.Cog.listener()
    async def on_wavelink_track_end(self, player, track, *args, **kwargs):
        self.bot.dispatch("dismusic_track_end", player, track)
        await self.handle_end_stuck_exception(player, track)

    @commands.Cog.listener()
    async def on_wavelink_track_exception(self, player, track, *args, **kwargs):
        self.bot.dispatch("dismusic_track_exception", player, track)
        await self.handle_end_stuck_exception(player, track)

    @commands.Cog.listener()
    async def on_wavelink_track_stuck(self, player, track, *args, **kwargs):
        self.bot.dispatch("dismusic_track_stuck", player, track)
        await self.handle_end_stuck_exception(player, track)

    @commands.Cog.listener()
    async def on_command_error(self, ctx, error):
        errors = (
            InvalidLoopMode,
            MustBeSameChannel,
            NotConnectedToVoice,
            PlayerNotConnected,
            NothingIsPlaying,
            NotEnoughSong,
        )

        if isinstance(error, errors):
            await ctx.send(error)
        else:
            pass



import asyncio
import discord
from discord import Embed, Color
import async_timeout
import wavelink
from discord import ClientException
from discord.ext import commands
from wavelink import (
    LavalinkException,
    LoadTrackError,
    SoundCloudTrack,
    YouTubeMusicTrack,
    YouTubePlaylist,
    YouTubeTrack,
)
from wavelink.ext import spotify
from wavelink.ext.spotify import SpotifyTrack

from ._classes import Provider
from .checks import voice_channel_player, voice_connected
from .errors import MustBeSameChannel
from .paginator import Paginator
from .player import DisPlayer


class Music(commands.Cog):

    def __init__(self, bot):
        self.bot: commands.Bot = bot
        self.bot.loop.create_task(self.start_nodes())

    def get_nodes(self):
        return sorted(wavelink.NodePool._nodes.values(), key=lambda n: len(n.players))

    async def play_track(self, ctx: commands.Context, query: str, provider=None):
        player: DisPlayer = ctx.voice_client

        if ctx.author.voice.channel.id != player.channel.id:
            raise MustBeSameChannel(
                "You must be in the same voice channel as the player."
            )

        track_providers = {
            "yt": YouTubeTrack,
            "ytpl": YouTubePlaylist,
            "ytmusic": YouTubeMusicTrack,
            "soundcloud": SoundCloudTrack,
            "spotify": SpotifyTrack,
        }


        query = query.strip("<>")
        msg = await ctx.send(f"Searching for `{query}` <:R_:961248893176283146>")

        track_provider = provider if provider else player.track_provider

        if track_provider == "yt" and "playlist" in query:
            provider = "ytpl"

        provider: Provider = (
            track_providers.get(provider)
            if provider
            else track_providers.get(player.track_provider)
        )

        nodes = self.get_nodes()
        tracks = list()

        for node in nodes:
            try:
                with async_timeout.timeout(20):
                    tracks = await provider.search(query, node=node)
                    break
            except asyncio.TimeoutError:
                self.bot.dispatch("dismusic_node_fail", node)
                wavelink.NodePool._nodes.pop(node.identifier)
                continue
            except (LavalinkException, LoadTrackError):
                continue

        if not tracks:
            return await msg.edit("<:loi2:961250273412677652> No Song Or Track Found With Your Link Give")

        if isinstance(tracks, YouTubePlaylist):
            tracks = tracks.tracks
            for track in tracks:
                await player.queue.put(track)
                omg = discord.Embed(color=discord.Color.yellow())
                omg.add_field(name=f'Music Commands !', value=f'Add Song To Queue : `{len(tracks)}`')
                omg.set_image(url=f'https://gifimage.net/wp-content/uploads/2017/10/music-equalizer-gif-7.gif')
            await msg.edit(embed=omg)
        else:
            track = tracks[0]

        embed = discord.Embed(color=discord.Color.green())
        embed.add_field(name=f'Music Commands !', value=f'Add Song To Queue : `{track.title}`')
        embed.set_image(url=f'https://gifimage.net/wp-content/uploads/2017/10/music-equalizer-gif-7.gif')

        await msg.edit(embed=embed)
        await player.queue.put(track)

        if not player.is_playing():
            await player.do_next()

    async def start_nodes(self):
        await self.bot.wait_until_ready()
        spotify_credential = getattr(
            self.bot, "spotify_credentials", {"client_id": "", "client_secret": ""}
        )

        for config in self.bot.lavalink_nodes:
            try:
                node: wavelink.Node = await wavelink.NodePool.create_node(
                    bot=self.bot,
                    **config,
                    spotify_client=spotify.SpotifyClient(**spotify_credential),
                )
                print(f"[Lkinght Music] INFO - Created node: {node.identifier}")
            except Exception:
                print(
                    f"[Lkinght Music] ERROR - Failed to create node {config['host']}:{config['port']}"
                )

    @commands.command(aliases=["con"])
    @voice_connected()
    async def connect(self, ctx: commands.Context):
        if ctx.voice_client:
            return

        embed = discord.Embed(color=discord.Color.green())
        embed.add_field(name=f'Music Commands !', value=f'Connecting To `{ctx.author.voice.channel}`')
        embed.set_image(url=f'https://cdn.discordapp.com/avatars/742672021422342165/d26bfcf7495aada8ed9294d49933eb6a.png')

        msg = await ctx.send(embed=embed)

        try:
            player: DisPlayer = await ctx.author.voice.channel.connect(cls=DisPlayer)
            self.bot.dispatch("dismusic_player_connect", player)
        except (asyncio.TimeoutError, ClientException):
            return await msg.edit(content="Failed to connect to voice channel.")

        player.bound_channel = ctx.channel
        player.bot = self.bot
        abc = discord.Embed(color=discord.Color.green())
        abc.add_field(name=f'Music Commands !', value=f'Connected To `{ctx.author.voice.channel}`')
        abc.set_image(url=f'https://cdn.discordapp.com/avatars/742672021422342165/d26bfcf7495aada8ed9294d49933eb6a.png')
        await msg.edit(embed=abc)

    @commands.group(aliases=["p"], invoke_without_command=True)
    @voice_connected()
    async def play(self, ctx: commands.Context, *, query: str):
        await ctx.invoke(self.connect)
        await self.play_track(ctx, query)

    @play.command(aliases=["yt"])
    @voice_connected()
    async def youtube(self, ctx: commands.Context, *, query: str):
        await ctx.invoke(self.connect)
        await self.play_track(ctx, query, "yt")

    @play.command(aliases=["ytmusic"])
    @voice_connected()
    async def youtubemusic(self, ctx: commands.Context, *, query: str):
        await ctx.invoke(self.connect)
        await self.play_track(ctx, query, "ytmusic")

    @play.command(aliases=["sc"])
    @voice_connected()
    async def soundcloud(self, ctx: commands.Context, *, query: str):
        await ctx.invoke(self.connect)
        await self.play_track(ctx, query, "soundcloud")

    @play.command(aliases=["sp"])
    @voice_connected()
    async def spotify(self, ctx: commands.Context, *, query: str):
        await ctx.invoke(self.connect)
        await self.play_track(ctx, query, "spotify")

    @commands.command(aliases=["vol"])
    @voice_channel_player()
    async def volume(self, ctx: commands.Context, vol: int, forced=False):
        player: DisPlayer = ctx.voice_client
        if vol < 0:
            return await ctx.send("⛔ | Volume Can't Be Less Than 0 ")

        if vol > 100 and not forced:
            return await ctx.send("⛔ | Volume Can't Greater Than 100")

        await player.set_volume(vol)
        await ctx.send(f"✅ | Volume Set To {vol} :loud_sound:")
        


    @commands.command(aliases=["disconnect", "dc"])
    @voice_channel_player()
    async def stop(self, ctx: commands.Context):
        player: DisPlayer = ctx.voice_client

        await player.destroy()

        tcm = discord.Embed(color=discord.Color.red())
        tcm.add_field(name=f'Music Commands !', value=f'Stop Play Music At `{ctx.author.voice.channel}`')
        tcm.set_image(url=f'https://th.bing.com/th/id/R.205a21c7900450742895142f5c1dbd72?rik=04APERZMDhombw&pid=ImgRaw&r=0')
        await ctx.send(embed=tcm)
        self.bot.dispatch("dismusic_player_stop", player)

    @commands.command()
    @voice_channel_player()
    async def pause(self, ctx: commands.Context):
        player: DisPlayer = ctx.voice_client

        if player.is_playing():
            if player.is_paused():
                return await ctx.send("❌ | Player is already paused.")

            await player.set_pause(pause=True)
            self.bot.dispatch("dismusic_player_pause", player)
            uml = discord.Embed(color=discord.Color.red())
            uml.add_field(name=f'Music Commands !', value=f'Pause Music At `{ctx.author.voice.channel}`')
            uml.set_image(url=f'https://th.bing.com/th/id/R.42bd92c08b3bab7a6841d60f921c9e0e?rik=98utNBGLb2F6cQ&pid=ImgRaw&r=0')
            return await ctx.send(embed=uml)

        await ctx.send("Player is not playing anything.")

    @commands.command()
    @voice_channel_player()
    async def resume(self, ctx: commands.Context):
        player: DisPlayer = ctx.voice_client

        if player.is_playing():
            if not player.is_paused():
                return await ctx.send("Player is already playing.")

            await player.set_pause(pause=False)
            self.bot.dispatch("dismusic_player_resume", player)
            return await ctx.send("Resumed :musical_note: ")

        await ctx.send("Player is not playing anything.")

    @commands.command()
    @voice_channel_player()
    async def skip(self, ctx: commands.Context):
        player: DisPlayer = ctx.voice_client

        if player.loop == "CURRENT":
            player.loop = "NONE"

        await player.stop()

        self.bot.dispatch("dismusic_track_skip", player)
        await ctx.send("Skipped :track_next:")

    @commands.command()
    @voice_channel_player()
    async def loop(self, ctx: commands.Context, loop_type: str = None):
        player: DisPlayer = ctx.voice_client

        result = await player.set_loop(loop_type)
        await ctx.send(f"Loop has been set to {result} :repeat: ")

    @commands.command(aliases=["q"])
    @voice_channel_player()
    async def queue(self, ctx: commands.Context):
        player: DisPlayer = ctx.voice_client

        if len(player.queue._queue) < 1:
            omgo = discord.Embed(color=discord.Color.red())
            omgo.add_field(name=f'Music Commands !', value=f'Nothing In Queue `{ctx.author}`')
            omgo.set_image(url=f'https://th.bing.com/th/id/R.42bd92c08b3bab7a6841d60f921c9e0e?rik=98utNBGLb2F6cQ&pid=ImgRaw&r=0')
            return await ctx.send(embed=omgo)

        paginator = Paginator(ctx, player)
        await paginator.start()

    @commands.command(aliases=["np"])
    @voice_channel_player()
    async def nowplaying(self, ctx: commands.Context):
        player: DisPlayer = ctx.voice_client
        await player.invoke_player(ctx)



from importlib.util import module_from_spec, spec_from_file_location
from pathlib import Path

from setuptools import find_packages, setup

classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "Framework :: aiohttp",
    "Operating System :: MacOS",
    "Operating System :: POSIX :: Linux",
    "Operating System :: Microsoft :: Windows",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
]

with open("mikomusic/requirements.txt") as f:
    requirements = f.read().splitlines()

current_directory = Path(__file__).parent.resolve()
long_description = (current_directory / "README.md").read_text(encoding="utf-8")

version_path = current_directory / "mikomusic" / "_version.py"
module_spec = spec_from_file_location(version_path.name[:-3], version_path)
version_module = module_from_spec(module_spec)
module_spec.loader.exec_module(version_module)

setup(
    name="Kinop",
    version=version_module.__version__,
    description="Music cog for discord bots. Supports YouTube, YoutubeMusic, SoundCloud and Spotify.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    classifiers=classifiers,
    keywords="musicbot",
    packages=find_packages(),
    python_requires=">=3.7, <4",
    install_requires=requirements,
    project_urls={
        "Bug Reports": "https://github.com",
        "Source": "https://github.com",
    },
)



import asyncio
import os

import async_timeout
import discord
from discord.ext import commands
from wavelink import Player

from .errors import InvalidLoopMode, NotEnoughSong, NothingIsPlaying


class DisPlayer(Player):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.queue = asyncio.Queue()
        self.loop = "NONE"  # CURRENT, PLAYLIST
        self.bound_channel = None
        self.track_provider = "yt"

    async def destroy(self) -> None:
        self.queue = None

        await super().stop()
        await super().disconnect()

    async def do_next(self) -> None:
        if self.is_playing():
            return

        timeout = int(os.getenv("DISMUSIC_TIMEOUT", 300))

        try:
            with async_timeout.timeout(timeout):
                track = await self.queue.get()
        except asyncio.TimeoutError:
            if not self.is_playing():
                await self.destroy()

            return

        self._source = track
        await self.play(track)
        self.client.dispatch("dismusic_track_start", self, track)
        await self.invoke_player()

    async def set_loop(self, loop_type: str) -> None:
        valid_types = ["NONE", "CURRENT", "PLAYLIST"]

        if not self.is_playing():
            raise NothingIsPlaying("Player is not playing any track. Can't loop")

        if not loop_type:
            if valid_types.index(self.loop) >= 2:
                loop_type = "NONE"
            else:
                loop_type = valid_types[valid_types.index(self.loop) + 1]

            if loop_type == "PLAYLIST" and len(self.queue._queue) < 1:
                loop_type = "NONE"

        if loop_type.upper() == "PLAYLIST" and len(self.queue._queue) < 1:
            raise NotEnoughSong(
                "There must be 2 songs in the queue in order to use the PLAYLIST loop"
            )

        if loop_type.upper() not in valid_types:
            raise InvalidLoopMode("Loop type must be `NONE`, `CURRENT` or `PLAYLIST`.")

        self.loop = loop_type.upper()

        return self.loop

    async def invoke_player(self, ctx: commands.Context = None) -> None:
        track = self.source

        if not track:
            raise NothingIsPlaying("Player is not playing anything.")

        embed = discord.Embed(
            title=track.title, url=track.uri, color=discord.Color(0x2F3136)
        )
        embed.set_author(
            name=track.author,
            url=track.uri,
            icon_url=self.client.user.display_avatar.url,
        )
        try:
            embed.set_thumbnail(url=track.thumb)
        except AttributeError:
            embed.set_thumbnail(
                url="https://cdn.discordapp.com/attachments/776345413132877854/940540758442795028/unknown.png"
            )
        embed.add_field(
            name="Length",
            value=f"{int(track.length // 60)}:{int(track.length % 60)}",
        )
        embed.add_field(name="Looping", value=self.loop)
        embed.add_field(name="Volume", value=self.volume)

        next_song = ""

        if self.loop == "CURRENT":
            next_song = self.source.title
        else:
            if len(self.queue._queue) > 0:
                next_song = self.queue._queue[0].title

        if next_song:
            embed.add_field(name="Next Song", value=next_song, inline=False)

        if not ctx:
            return await self.bound_channel.send(embed=embed)

        await ctx.send(embed=embed)
